from sobeit.imgui import ImGui, ImColor

from modsa_lib.consts.sobeit import EVENT_GAME_INIT
from modsa_lib.core.event import Event
from modsa_lib.core.sobeit_api import PlayerAPI
from modsa_lib.utils.checker import BaseChecker, Checker
from modsa_lib.consts.raknet import *
from modsa_lib.imgui.utils import ImGuiUtils
from modsa_lib.core.bitstream import parse_rpc


class JoinQuitChecker(BaseChecker):
    config_path = 'modsa_basic/checker_join_quit.json'

    def __init__(self):
        super().__init__()
        self.player_join = ""
        self.player_quit = ""

    def get_default_config(self) -> dict:
        config_data = super().get_default_config()
        config_data.update(
            {
                "pos_x": 0.001,
                "pos_y": 0.93,
            }
        )
        return config_data

    def render(self):
        ImGui.PushFont(self.window_font)
        ImGui.TextColored(ImColor(30, 255, 30), self.player_join)
        ImGui.TextColored(ImColor(255, 90, 90), self.player_quit)
        ImGui.PopFont()

    def handle_join_quit_rpc(self, bs_data, rpc_id):
        data = parse_rpc(bs_data, rpc_id)
        if rpc_id == RPC_ServerJoin:
            self.player_join = f' {data["name"]} [{data["id"]}]'
        else:
            self.player_quit = f' {PlayerAPI.get_name(data["id"])} [{data["id"]}]'


class AdminChecker(Checker):
    cmd_add_name = 'addadm'
    cmd_del_name = 'deladm'
    config_path = 'modsa_basic/checker_admin.json'
    players_json_path = 'mod_sa/data/checker_admins.json'

    def render_title(self):
        ImGui.TextColored(ImColor(0, 200, 0), f"Администраторы ({self.online_count})")

    def get_default_config(self) -> dict:
        config_data = super().get_default_config()
        config_data.update(
            {
                "pos_x": 0.55,
                "pos_y": 0.963,
            }
        )
        return config_data


class PlayerChecker(Checker):
    cmd_add_name = 'addplayer'
    cmd_del_name = 'delplayer'
    config_path = 'modsa_basic/checker_player.json'
    players_json_path = 'mod_sa/data/checker_players.json'

    def render_title(self):
        ImGui.TextColored(ImColor(255, 240, 61), f"Игроки ({self.online_count})")

    def get_default_config(self) -> dict:
        config_data = super().get_default_config()
        config_data.update(
            {
                "pos_x": 0.733,
                "pos_y": 0.963,
            }
        )
        return config_data


class TempChecker(Checker):
    cmd_add_name = 'addtemp'
    cmd_del_name = 'deltemp'
    config_path = 'modsa_basic/checker_temp.json'

    def render_title(self):
        ImGui.TextColored(ImColor(0, 180, 255), f"Временный ({self.online_count})")

    def get_default_config(self) -> dict:
        config_data = super().get_default_config()
        config_data.update(
            {
                "pos_x": 0.65,
                "pos_y": 0.963,
            }
        )
        return config_data


join_quit_checker = JoinQuitChecker()
admin_checker = AdminChecker()
player_checker = PlayerChecker()
temp_checker = TempChecker()


@Event.register(EVENT_GAME_INIT)
def on_game_init():
    join_quit_checker.setup()
    admin_checker.setup()
    player_checker.setup()
    temp_checker.setup()


@Event.register_rpc(RPC_InitGame)
def hook_init_game(bs_data, rpc_id):
    player_name = PlayerAPI.get_name()
    bs_data.IgnoreBits(104)
    player_id = bs_data.Read(TYPE_INT16)
    admin_checker.set_player_online(player_name, player_id)
    player_checker.set_player_online(player_name, player_id)


def render_checkers_config(rendering: bool):
    join_quit_checker.edit_mode = rendering
    admin_checker.edit_mode = rendering
    player_checker.edit_mode = rendering
    temp_checker.edit_mode = rendering

    if not rendering:
        join_quit_checker.config.save()
        admin_checker.config.save()
        player_checker.config.save()
        temp_checker.config.save()
        return

    ImGuiUtils.header(" --- [Чекер зашедших / вышедших] ---")
    if ImGui.Checkbox("Включить##joinquit", join_quit_checker.imgui_config['enabled']):
        join_quit_checker.config['enabled'] = join_quit_checker.imgui_config['enabled'].v
        join_quit_checker.start() if join_quit_checker.config['enabled'] else join_quit_checker.stop()

    ImGui.Text("Размер шрифта:")
    if ImGui.SliderFloat("##joinquit_font_scale", join_quit_checker.imgui_config['font_scale'], 0.3, 5.0):
        join_quit_checker.config["font_scale"] = join_quit_checker.imgui_config['font_scale'].v
        join_quit_checker.font_scale = join_quit_checker.imgui_config['font_scale'].v
    checkers = {
        " --- [Чекер администрации] ---": admin_checker,
        " --- [Чекер игроков] ---": player_checker,
        " --- [Временный чекер] ---": temp_checker,
    }

    for title, checker in checkers.items():
        ImGuiUtils.header(title)
        if ImGui.Checkbox(f"Включить##{checker.__class__.__name__}", checker.imgui_config['enabled']):
            checker.config["enabled"] = checker.imgui_config['enabled'].v
            checker.start() if checker.config["enabled"] else checker.stop()

        if ImGui.Checkbox(
            f"Отображение сверху вниз##{checker.__class__.__name__}", checker.imgui_config['direction_down']
        ):
            checker.config["direction_down"] = checker.imgui_config['direction_down'].v

        ImGui.SameLine()
        ImGuiUtils.help_text(
            "При отображении сверху вниз окно чекера не\nбудет менять местоположения (Окно будет увеличиваться снизу)"
        )
        if ImGui.Checkbox(
            f"Динамическое изменение размера окна##{checker.__class__.__name__}", checker.imgui_config['autosize']
        ):
            checker.config["autosize"] = checker.imgui_config['autosize'].v

        ImGui.Text("Размер шрифта:")
        if ImGui.SliderFloat(
            f"##{checker.__class__.__name__}_font_scale", checker.imgui_config['font_scale'], 0.3, 5.0
        ):
            checker.config["font_scale"] = checker.imgui_config['font_scale'].v
