from sobeit.imgui import ImGui
from typing import Optional

from modsa_lib.consts.keycombo import *
from modsa_lib.consts.sobeit import *
from modsa_lib.core import Chat
from modsa_lib.core.handlers import KeycomboHandler, HandlersManagementMixin
from modsa_lib.imgui.utils import ImGuiUtils


class KeycomboCreator(HandlersManagementMixin):
    """
    Класс считывания комбинаций клавиш (для настроек)
    """

    def __init__(self, print_help_text=True, **kwargs):
        self.handlers = [
            KeycomboHandler("", self.hook_keys, KEYCOMBO_DOWN_PRESSED),
            KeycomboHandler("", self.hook_keys, KEYCOMBO_DOWN_RELEASED),
        ]
        self.__keycombo = ""
        self.__keycombo_temp = {}
        self.__cancel = False
        self.__clear_keycombo = False
        self.__print_text = print_help_text
        self.kwargs = kwargs

    def hook_keys(self, keys, action_type):
        if action_type == KEYCOMBO_DOWN_PRESSED:
            if keys == KEY_LBUTTON:
                # Отключение клавиши
                self.reset()
                self.__clear_keycombo = True
                self.stop()
                if self.__print_text:
                    Chat.print_text("Клавиша отключена")
            elif keys == KEY_RBUTTON:
                # Отмена
                self.reset()
                self.__cancel = True
                self.stop()
                if self.__print_text:
                    Chat.print_text("Операция отменена")
            else:
                self.__keycombo_temp[keys] = False
        elif keys in self.__keycombo_temp:
            self.__keycombo_temp[keys] = True

        if self.__keycombo_temp and all(self.__keycombo_temp.values()):
            self.__keycombo = ' '.join(self.__keycombo_temp.keys())
            self.stop()
            if self.__print_text:
                Chat.print_text("Клавиша назначена")

    @property
    def keycombo(self):
        return self.__keycombo

    def is_complete(self):
        """ Завершил ли обработчик работу """
        return not self.is_running() and any(
            [self.__keycombo, self.__cancel, self.__clear_keycombo]
        )

    def is_cancel(self):
        """ Выбрана отмена настройки """
        return self.__cancel

    def is_clear_keycombo(self):
        """ Выбрано удаление клавиши """
        return self.__clear_keycombo

    def after_handlers_start(self):
        if self.__print_text:
            Chat.print_text(">>> Нажмите желаемую комбинацию клавиш", "FF0000")
            Chat.print_text(
                ">>> Для отключения клавиши нажмите \"Левую кнопку мыши\"", "FF0000"
            )
            Chat.print_text(
                ">>> Для отмены ввода нажмите \"Правую кнопку мыши\"", "FF0000"
            )

        self.reset()
        return True

    def reset(self):
        self.__keycombo = ""
        self.__keycombo_temp.clear()
        self.__cancel = False
        self.__clear_keycombo = False


class KeycomboPressed(HandlersManagementMixin):
    """
    Класс обработки нажатых клавиш
    """

    def __init__(self, keys: str, callback: object):
        """
        Args:
            keys: строка с клавишами
        """
        self.keys = keys
        self.callback = callback
        if keys:
            self.handlers = [KeycomboHandler(self.keys, self.callback)]

    def replace_keys(self, new_keys):
        was_run = self.is_running()
        self.keys = new_keys

        if was_run:
            self.stop()
        if new_keys:
            self.handlers = [KeycomboHandler(self.keys, self.callback)]
        if was_run:
            self.start()


class KeycomboUpdaterMixin:
    """
    Класс для обновления группы клавиш и отображении их в настройках
    keycombo_handlers должен иметь следующий вид:
    {
        'my_key_1': KeycomboPressed(...),
        'my_key_2': KeycomboPressed(...),
        ...
    }
    Пример: goto к себе в srp_basic/common.py
    """

    keycombo_handlers = None
    __key_update = None
    __new_keycombo = None

    def set_keycombo_handlers(self, handlers: dict, auto_start=True):
        self.keycombo_handlers = handlers

        if auto_start:
            for handler in list(self.keycombo_handlers.values()):
                handler.start()

    def get_new_keycombo(self):
        """Получение нажатых клавиш после настройки"""
        return self.__new_keycombo

    def imgui_keycombo(self, key: str, label: Optional[str] = None) -> bool:
        """Рендеринг настройки клавиши"""
        if not self.keycombo_handlers or key not in self.keycombo_handlers:
            return False

        if label:
            ImGui.Text(f'{label}:')
        if ImGuiUtils.keycombo_settings(self.keycombo_handlers[key].keys, key):
            self.__key_update = key
            keycombo_creator.start()

        return self.__key_update == key and self._check_complete_keycombo_updater()

    def _check_complete_keycombo_updater(self) -> bool:
        """Проверка завершения настройки клавиш"""
        if keycombo_creator.is_complete() and not keycombo_creator.is_cancel():
            self.keycombo_handlers[self.__key_update].replace_keys(
                keycombo_creator.keycombo
            )
            self.__new_keycombo = keycombo_creator.keycombo
            self.__key_update = None
            keycombo_creator.reset()
            return True
        return False


keycombo_creator = KeycomboCreator()
