from functools import cached_property
from typing import Optional

from sobeit import player_api

from modsa_lib.core.sobeit_api import PlayerAPI
from modsa_lib.settings import MAX_PLAYERS
from modsa_lib.utils.common import TextUtils


class Player:
    def __init__(self, playerid: Optional[int] = None):
        if playerid is None:
            playerid = PlayerAPI.get_id()  # local playerid
        self.id = playerid
        self.is_valid()  # cache player name

    @cached_property
    def name(self):
        player_name = player_api.get_name(self.id)
        return TextUtils.bytes_to_text(player_name) if player_name else None

    @property
    def ping(self) -> Optional[int]:
        return player_api.get_ping(self.id)

    @property
    def score(self) -> Optional[int]:
        return player_api.get_score(self.id)

    @property
    def position(self) -> Optional[tuple]:
        return player_api.get_position(self.id)

    @property
    def health(self) -> Optional[int]:
        return player_api.get_health(self.id)

    def is_valid(self) -> bool:
        return player_api.is_valid(self.id) and self.name == player_api.get_name(self.id)

    def is_near(self) -> bool:
        return player_api.is_near(self.id)

    def get_bone_position(self, bone: int) -> Optional[tuple]:
        return player_api.get_bone_position(self.id, bone)

    def __bool__(self):
        return self.is_valid()

    def __int__(self):
        return self.id


class PlayerFilter:
    def __init__(self):
        self.filters = []

    def in_stream(self):
        self.filters.append(lambda x: player_api.is_near(x))
        return self

    def not_in_stream(self):
        self.filters.append(lambda x: not player_api.is_near(x))
        return self

    def afk(self):
        self.filters.append(lambda x: player_api.is_afk(x))
        return self

    def not_afk(self):
        self.filters.append(lambda x: not player_api.is_afk(x))
        return self


class PlayerIter(PlayerFilter):
    def __iter__(self):
        for playerid in range(0, MAX_PLAYERS):
            if player_api.is_valid(playerid) and all(f(playerid) for f in self.filters):
                yield Player(playerid)
