from sobeit.imgui import ImVec2
from sobeit.raknet import BitStream
from sobeit.render_api import D3DXVECTOR3, RenderAPI as render_api
from typing import Optional

from sobeit import command_api, player_api, script_api, event_api

from modsa_lib.consts.sobeit import DIALOG_MSGBOX, KEYCOMBO_DOWN_PRESSED
from modsa_lib.utils.common import TextUtils


class CommandAPI:
    @classmethod
    def register(cls, cmd: str, func: object) -> Optional[int]:
        """
        Регистрация новой команды
        Args:
            cmd: название команды
            func: вызываемая функция
        Return:
            ID обработчика в случае успеха, иначе None

        В вызываемую функцию будут переданы 2 переменные:
            args: введенные аргументы
            cmd: название команды без слеша
        """
        return command_api.raw_register(TextUtils.text_to_bytes(cmd), func)

    @classmethod
    def remove(cls, event_handler: int) -> bool:
        """
        Удаление обработчика команды
        Args:
            event_handler: идентификатор обработчика
        Return:
            True в случае успеха, иначе False
        """
        return command_api.remove(event_handler)

    @classmethod
    def bind(
        cls,
        keycombo: str,
        func: object,
        action_type: int = KEYCOMBO_DOWN_PRESSED,
        send_to_game: bool = False,
    ) -> Optional[int]:
        """
        Регистрация обработчика нажатия клавиш
        Args:
            keycombo: клавиша(и)
            func: вызываемая функция
            action_type: тип нажатия (Из Keycombo action)
            send_to_game: отправить нажатие игре
        Return:
            ID обработчика в случае успеха, иначе None

        При send_to_game = False игра не будет реагировать на нажатие выбранных клавиш
        (кроме клавиш открытия / закрытия чата)
        Они заработают только при unbind'е обработчика
        """
        return command_api.bind(keycombo, func, send_to_game, action_type)

    @classmethod
    def unbind(cls, event_handler: int) -> bool:
        """
        Удаление обработчика нажатия клавиш
        Args:
            event_handler: идентификатор обработчика
        Return:
            True в случае успеха, иначе False
        """
        return command_api.unbind(event_handler)


class GuiAPI:
    @classmethod
    def show_samp_dialog(
        cls,
        callback=None,
        dialog_type: int = DIALOG_MSGBOX,
        caption: str = 'Caption',
        text: str = 'Text',
        button1: str = 'OK',
        button2: str = '',
        send: bool = False,
        dialog_id: int = 0,
    ):
        command_api.raw_show_samp_dialog(
            callback,
            dialog_type,
            TextUtils.text_to_bytes(caption),
            TextUtils.text_to_bytes(text),
            TextUtils.text_to_bytes(button1),
            TextUtils.text_to_bytes(button2),
            send,
            dialog_id,
        )

    @classmethod
    def samp_toggle_cursor(cls, state: bool):
        """Активация / Выключение отображения курсора"""
        command_api.toggleSAMPCursor(int(state))


class PlayerAPI:
    @classmethod
    def is_near(cls, playerid: int) -> bool:
        """
        Находится ли игрок с указанным playerid в зоне прорисовки
        Args:
            playerid: ид игрока
        Return:
            True - если игрок в зоне прорисовки, иначе False
        """
        return player_api.is_near(playerid) if isinstance(playerid, int) else None

    @classmethod
    def is_valid(cls, playerid: int) -> bool:
        """
        Является ли игрок с указанным ИД валидным (в игре)
        Args:
            playerid: ид игрока
        Return:
            True - если игрок валидный, иначе False
        """
        return player_api.is_valid(playerid) if isinstance(playerid, int) else None

    @classmethod
    def is_afk(cls, playerid: int) -> bool:
        """
        Находится ли игрок в АФК
        Args:
            playerid: ид
        Return:
            true если AFK, в остальных случаях false
        """
        return player_api.is_afk(playerid) if isinstance(playerid, int) else None

    @classmethod
    def get_players(cls, stream: bool = False) -> list:
        """
        Получение списка игроков. Каждый игрок представлен
        в виде словаря с элементами: id, name, score, ping

        Args:
            stream: если True, то вернутся только игроки в зоне прорисовки, иначе все
        Return:
            Список игроков
        """
        players = player_api.get_players(stream)
        for player in players:
            player['name'] = TextUtils.bytes_to_text(player['name'])
        return players

    @classmethod
    def get_local_player(cls) -> dict:
        """
        Получение данных текущего игрока
        Return:
            Словаоь, содержащий id, name, score и ping текущего игрока
        Примечание: вернёт пустой словарь, если игра еще не инициализирована
        """
        player = player_api.get_local_player()
        if player:
            player['name'] = TextUtils.bytes_to_text(player['name'])
        return player

    @classmethod
    def get_name(cls, playerid: int = -1) -> Optional[str]:
        """
        Получить ник игрока
        Args:
            playerid: ид игрока (-1 для текущего игрока)
        Return:
            Ник игрока, если игрок валидный. Иначе None
        """
        return TextUtils.bytes_to_text(player_api.get_name(playerid)) if cls.is_valid(playerid) else None

    @classmethod
    def get_id(cls, playername: str = '') -> Optional[int]:
        """
        Получить ид игрока по нику
        Args:
            playername: ник (пустая строка - ид текущего игрока)
        Return:
            Ид игрока, если игрок найден. Иначе None
        """
        if playername == '' or cls.get_name() == playername:
            return cls.get_local_player()['id']

        players = cls.get_players()
        for player in players:
            if player['name'] == playername:
                return player['id']

    @classmethod
    def get_position(cls, playerid: int = -1) -> Optional[list]:
        """
        Получить координаты игрока
        Args:
            playerid: ид игрока (-1 для текущего игрока)
        Return:
            Координаты x, y, z в виде списка при успехе, иначе None
        """
        return player_api.get_position(playerid) if isinstance(playerid, int) else None

    @classmethod
    def get_health(cls, playerid: int = -1) -> Optional[int]:
        """
        Получить здоровье игрока
        Args:
            playerid: ид игрока (-1 для текущего игрока)
        Return:
            Уровень HP при успехе, иначе None
        """
        return player_api.get_health(playerid) if isinstance(playerid, int) else None

    @classmethod
    def get_bone_position(cls, bone: int, playerid: int = -1) -> Optional[tuple]:
        """
        Получение координат указанной кости игрока
        Args:
            bone: ид кости (см. modsa_lib.consts.sobeit.Bones)
            playerid: ид игрока (-1 для текущего игрока)
        Return:
            (x, y, z), либо None, при возникновении ошибки.
        """
        return player_api.get_bone_position(bone, playerid) if isinstance(playerid, int) else None

    @classmethod
    def get_weapon(cls, playerid: int = -1):
        """
        Получение текущего оружия в руках у игрока
        Args:
            playerid: ид игрока (-1 для текущего игрока)
        Return:
            ID оружия, либо None, при возникновении ошибки.
        """
        return player_api.get_weapon(playerid) if isinstance(playerid, int) else None


class __RenderAPI:
    @property
    def display_width(self) -> float:
        # До загрузки игрового мира возвращает 0
        return render_api.displayWidth

    @property
    def display_height(self) -> float:
        # До загрузки игрового мира возвращает 0
        return render_api.displayHeight

    def parse_pos(self, x, y) -> ImVec2:
        if 0.0 < x < 1.0:
            x = int(x * self.display_width)
        elif x < 0:
            x = self.display_width + x

        if 0.0 < y < 1.0:
            y = int(y * self.display_height)
        elif y < 0:
            y = self.display_height + y

        return ImVec2(x, y)

    @classmethod
    def register(cls, func: object) -> int:
        """
        Регистрация рендера.
        Args:
            func: вызываемая функция
        Return:
            ID обработчика
        """
        return render_api.register(func)

    @classmethod
    def remove(cls, event_handler: int) -> bool:
        """
        Удаление рендера
        Args:
            event_handler: ID обработчика
        Return:
            True в случае успеха, иначе False
        """
        return render_api.remove(event_handler)

    @classmethod
    def cheat_state_text(cls, text: str):
        """
        Показ текста в правой верхней части экрана
        """
        render_api.raw_cheat_state_text(TextUtils.text_to_bytes(text))

    @classmethod
    def draw_line(cls, x: float, y: float, z: float, tx: float, ty: float, tz: float, color: int):
        return render_api.draw_line(x, y, z, tx, ty, tz, color)

    @classmethod
    def draw_sphere(cls, x: float, y: float, z: float, r: float, color: int):
        return render_api.draw_sphere(x, y, z, r, color)

    @classmethod
    def calc_screen_coors(cls, vec_world: D3DXVECTOR3) -> D3DXVECTOR3:
        return render_api.calc_screen_coors(vec_world)


RenderAPI = __RenderAPI()


class EventAPI:
    @classmethod
    def register(cls, event: int, func: object) -> int:
        """
        Регистрация обработчика событий собейта
        Args:
            event: код события
            func: вызываемая функция
        Return:
            идентификатор обработчика или 0, если возникла ошибка
        """
        return event_api.register(event, func)

    @classmethod
    def register_rpc(cls, event: int, func: object) -> int:
        """
        Регистрация обработчика входящих / исходящих пакетов RPC
        Args:
            event: тип пакета
            func: вызываемая функция
        Return:
            идентификатор обработчика

        В вызываемую функцию будут переданы:
            bs_data: объект BitStream, содержащий данные пакета
            event: тип пакета
        """
        return event_api.register_rpc(event, func)

    @classmethod
    def register_packet(cls, event: int, func: object, incomming: bool = True) -> int:
        """
        Регистрация обработчика входящих / исходящих пакетов
        Args:
            event: тип пакета
            func: вызываемая функция
            incomming: если True - обработка входящих пакетов, иначе исходящих
        Return:
            идентификатор обработчика

        В вызываемую функцию будут переданы:
            bs_data: объект BitStream, содержащий данные пакета
            event: тип пакета
        """
        return event_api.register_packet(event, func, incomming)

    @classmethod
    def remove(cls, event_handler: int) -> bool:
        """
        Удаление обработчика
        Args:
            event_handler: идентификатор обработчика
        Return:
            True в случае успеха, иначе False
        """
        return event_api.remove(event_handler)

    @classmethod
    def send_rpc(cls, event: int, bs_data: BitStream) -> bool:
        """
        Отправка RPC пакета
        Args:
            event: тип пакета
            bs_data: BitStream
        Return:
            True в случае успеха, иначе False
        """
        return event_api.send_rpc(event, bs_data)

    @classmethod
    def send_packet(cls, bs_data: BitStream) -> bool:
        """
        Отправка пакета
        Args:
            bs_data: BitStream
        Return:
            True в случае успеха, иначе False
        """
        return event_api.send_packet(bs_data)


class ScriptAPI:
    @classmethod
    def register(cls, data: dict):
        script_api.register(data)
