from sobeit.imgui import ImColor, ImGui, ImVec2, ImBool, ImFloat, ImString

from modsa_lib.core import Chat
from modsa_lib.core.command import CmdArgParser, Command, CmdArgPlayeridName, CmdArg
from modsa_lib.imgui.flags import *
from modsa_lib.utils.config import ConfigMixin
from modsa_lib.consts.raknet import *
from modsa_lib.imgui.fonts import IMGUI_DEFAULT_FONT_17
from modsa_lib.core.handlers import RPCHandler, RenderHandler, HandlersManagementMixin
from modsa_lib.imgui.utils import ImguiWindow
from modsa_lib.utils.json_helper import JsonFile
from modsa_lib.core.bitstream import parse_rpc
from modsa_lib.core.sobeit_api import PlayerAPI, RenderAPI
from modsa_lib.utils.common import ColorUtils


class BaseChecker(ConfigMixin, HandlersManagementMixin):
    """Базовый класс чекера"""

    players_json_path = None

    window_flags = ImGuiWindowFlags_NoDecoration | ImGuiWindowFlags_NoFocusOnAppearing
    window_alpha = 0.6
    window_font = IMGUI_DEFAULT_FONT_17
    edit_mode = False

    def __init__(self):
        self.__players = dict()
        self.__online_count = 0
        self.__line_height = 17.0

        self.load_config()
        self.load_players_from_json()

        if self.config['autosize']:
            self.window_flags |= ImGuiWindowFlags_AlwaysAutoResize

        self.size = ImVec2(
            self.config.get("size_x", 100),
            self.config.get("size_y", 100),
        )
        self.pos = None
        self.imgui_window = None
        self.default_text_color = ImColor(*ColorUtils.hex_to_rgb("2990ff"))

        self.handlers = [
            RPCHandler(RPC_ServerJoin, self.handle_join_quit_rpc),
            RPCHandler(RPC_ServerQuit, self.handle_join_quit_rpc),
            RenderHandler(self.pre_render),
        ]

        self.imgui_config = {
            "enabled": ImBool(self.config.get("enabled", True)),
            "font_scale": ImFloat(self.config['font_scale']),
            "direction_down": ImBool(self.config['direction_down']),
            "autosize": ImBool(self.config['autosize']),
        }

        self.imgui_settings_window = None
        self.settings_window_data = {}

    def setup(self):
        self.pos = RenderAPI.parse_pos(
            self.config.get("pos_x", 0), self.config.get("pos_y", 0)
        )
        self.imgui_window = ImguiWindow(
            f"##checker__{self.__class__.__name__}",
            pos=self.pos,
            size=self.size,
        )
        self.imgui_settings_window = ImguiWindow(
            f"##checker_settings__{self.__class__.__name__}",
            size=ImVec2(200, 150),
            disable_chat=True,
        )

        if self.config.get("enabled", True):
            self.start()

    def get_default_config(self) -> dict:
        return {
            'enabled': True,
            'autosize': True,
            'direction_down': False,
            'font_scale': 1.0,
            'pos_x': 0,
            'pos_y': 0,
            'size_x': 100,
            'size_y': 100,
        }

    def load_players_from_json(self):
        if not self.players_json_path:
            return

        file_data = JsonFile(self.players_json_path).data
        for name, data in file_data.items():
            self.add_player(name, None, data)

    def set_player_online(self, player_name: str, player_id: int):
        if player_name in self.__players:
            self.__online_count += 1
            self.__players[player_name]["id"] = player_id

    def set_player_offline(self, player_name: str):
        if player_name in self.__players:
            self.__online_count -= 1
            self.__players[player_name]["id"] = None

    @property
    def online_count(self):
        return self.__online_count

    def reload_players(self):
        self.clear_all_players()
        self.load_players_from_json()
        for player in PlayerAPI.get_players():
            self.set_player_online(player['name'], player['id'])

    def handle_join_quit_rpc(self, bs_data, packet_id):
        data = parse_rpc(bs_data, packet_id)
        if packet_id == RPC_ServerJoin:
            self.set_player_online(data["name"], data["id"])
        else:
            self.set_player_offline(PlayerAPI.get_name(data["id"]))

    def get_y_offset(self):
        return (
            self.online_count * self.__line_height
            if self.config['autosize'] and not self.config['direction_down']
            else 0
        )

    def update_settings(self):
        if self.edit_mode:
            self.pos = ImGui.GetWindowPos()
            if self.config['autosize'] and not self.config['direction_down']:
                self.pos.y += self.get_y_offset()
            else:
                window_size = ImGui.GetWindowSize()
                self.config["size_x"] = window_size.x
                self.config["size_y"] = window_size.y

            self.config["pos_x"] = self.pos.x / RenderAPI.display_width
            self.config["pos_y"] = self.pos.y / RenderAPI.display_height

    def render_settings_window(self):
        if not self.imgui_settings_window.is_open():
            return

        with self.imgui_settings_window:
            ImGui.Text(self.settings_window_data.get('player', '-'))
            color = self.settings_window_data['data']['color']
            text = ImString(self.settings_window_data['data'].get('text', ''))
            if ImGui.InputText("Текст", text):
                self.settings_window_data['data']['text'] = text.v
            ImGui.ColorEdit3("Цвет ника", color, ImGuiColorEditFlags_NoInputs)
            if ImGui.Button('Сохранить'):
                self.imgui_settings_window.hide()
                self.save_players()

    def pre_render(self):
        self.render_settings_window()
        ImGui.SetNextWindowBgAlpha(self.window_alpha)
        if not self.edit_mode:
            self.window_flags |= ImGuiWindowFlags_NoResize
            ImGui.SetNextWindowPos(ImVec2(self.pos.x, self.pos.y - self.get_y_offset()))

        if not self.config['autosize']:
            ImGui.SetNextWindowSize(self.size, ImGuiCond_FirstUseEver)
            if self.edit_mode:
                self.window_flags &= ~ImGuiWindowFlags_NoResize

        ImGui.PushStyleVar(ImGuiStyleVar_ItemSpacing, ImVec2(0, 0))
        ImGui.PushStyleVar(ImGuiStyleVar_WindowRounding, 10.0)

        self.imgui_window.flags = self.window_flags
        with self.imgui_window:
            ImGui.SetWindowFontScale(self.config['font_scale'])
            self.update_settings()
            self.render()

        ImGui.PopStyleVar(2)

    def render(self):
        ImGui.PushFont(self.window_font)
        self.__line_height = ImGui.GetTextLineHeightWithSpacing() - 0.4
        self.render_title()
        for name, data in self.__players.items():
            if data["id"] is None:
                continue

            if PlayerAPI.is_near(data["id"]):
                ImGui.TextColored(ImColor(0, 255, 255), "P ")
                ImGui.SameLine()

            self.render_player(name, data)
        ImGui.PopFont()

    def render_title(self):
        """Рендер шапки чекера"""
        pass

    def render_player(self, player_name: str, data: dict):
        ImGui.PushStyleColor(ImGuiCol_Text, data['color'].Value)
        if ImGui.Selectable(f"{player_name} [{data['id']}] "):
            self.settings_window_data = {
                'player': player_name,
                'data': data,
            }
            self.imgui_settings_window.show()
        ImGui.PopStyleColor()
        text = data.get('text')
        if text:
            ImGui.SameLine()
            ImGui.Text(text)

    def add_player(self, player_name, player_id=None, data=None) -> bool:
        if player_name in self.__players:
            return False

        if data is None:
            data = {'color': 'FFFFFF'}

        color = data.pop('color', 'FFFFFF')
        rgb_color = ColorUtils.hex_to_rgb(color)
        self.__players[player_name] = {
            "id": player_id,
            "color": ImColor(rgb_color[0], rgb_color[1], rgb_color[2]),
            **data,
        }
        if player_id is not None:
            self.set_player_online(player_name, player_id)

        return True

    def delete_player(self, player_name) -> bool:
        if player_name in self.__players:
            self.set_player_offline(player_name)
            self.__players.pop(player_name, None)
            return True
        return False

    def clear_all_players(self):
        self.__players.clear()
        self.__online_count = 0

    def save_players(self):
        if not self.players_json_path:
            return

        save_data = {}
        for name, player_data in self.__players.items():
            if player_data:
                save_data[name] = self.prepare_player_save_data(player_data)

        JsonFile(self.players_json_path, autoload=False).write(save_data)

    def prepare_player_save_data(self, player_data: dict):
        prepared_data = {}
        for key, value in player_data.items():
            if key == 'color':
                color = player_data["color"].Value
                prepared_data[key] = ColorUtils.rgb_to_hex(
                    color.x * 255, color.y * 255, color.z * 255
                )
            elif key == 'id':
                continue
            else:
                prepared_data[key] = value
        return prepared_data


class Checker(BaseChecker):
    cmd_add_name = None
    cmd_del_name = None

    __cmd_add = None
    __cmd_del = None

    def __init__(self):
        super().__init__()
        self.register_commands()

    def register_commands(self):
        if self.cmd_add_name and not self.__cmd_add:
            self.__cmd_add = Command(
                self.cmd_add_name,
                self.command_add,
                arg_parser=CmdArgParser(
                    {
                        'player': CmdArgPlayeridName(),
                        'text': CmdArg(all_text=True, required=False),
                    }
                ),
            )
        if self.cmd_del_name and not self.__cmd_del:
            self.__cmd_add = Command(
                self.cmd_del_name,
                self.command_delete,
                arg_parser=CmdArgParser({'player': CmdArgPlayeridName()}),
            )

    def command_add(self, args, cmd, parser_data):
        player_name = parser_data.get("player")
        text = parser_data.get("text")
        playerid = PlayerAPI.get_id(player_name)
        data = {'text': text} if text else None

        if self.add_player(player_name, playerid, data):
            self.save_players()
            Chat.print_text(f"{player_name} добавлен в чекер")
        else:
            Chat.print_text(f"{player_name} уже добавлен в чекер")

    def command_delete(self, args, cmd, parser_data):
        player_name = parser_data.get("player")
        if self.delete_player(player_name):
            self.save_players()
            Chat.print_text(f"{player_name} удален из чекера")
        else:
            Chat.print_text(f"{player_name} не найден в чекере")
