from copy import deepcopy

from sobeit import event_api
from sobeit.imgui import ImVec2, ImGui, ImBool, ImColor
from sobeit.raknet import BitStream

from modsa_lib.consts.raknet import *
from modsa_lib.core import RenderAPI
from modsa_lib.core.handlers import RenderHandler, PacketHandler, KeycomboHandler
from modsa_lib.core.bitstream import parse_packet
from modsa_lib.imgui.flags import *
from modsa_lib.utils.common import ColorUtils

recon_keys = ['w', 's', 'a', 'd', 'lshift', 'up', 'down', 'left', 'right', 'space']


class Recon:
    """
    Базовый класс рекона
    """

    def __init__(self, panel_template: dict, stats_template: dict):
        self.windows = {}
        self.stats = {}
        self.callbacks = {}
        self.player_name = ""
        self.player_id = -1
        self.recon_handlers = [
            RenderHandler(self.__render),
            PacketHandler(ID_SPECTATOR_SYNC, self._spec_hook, False),
        ]

        for key in recon_keys:
            self.recon_handlers.append(KeycomboHandler(key, self.__keycombo_callback))

        self._rendering = False
        self._change_mode = False
        self._active_window = ""
        self._main_window_name = "recon_main"
        self._windows_list = []
        self._panel_template = panel_template
        self._panel_flags = ImGuiWindowFlags_NoTitleBar | ImGuiWindowFlags_NoResize
        self._panel_font = None
        self._panel_font_size = panel_template.get('panel_font_size', 1.2)
        self._stats_template = stats_template
        self._stats_font = None

        self._init_panel_template(self._panel_template)
        self._init_stats_template()

    def show(self, change_mode=False) -> bool:
        if self._rendering:
            return False

        self._rendering = True
        self._change_mode = change_mode
        self._active_window = self._panel_template['name']
        self._main_window_name = self._active_window

        for handler in self.recon_handlers:
            handler.start()

        if self._change_mode:
            self._panel_flags = ImGuiWindowFlags_NoTitleBar
        else:
            self._panel_flags = ImGuiWindowFlags_NoTitleBar | ImGuiWindowFlags_NoResize | ImGuiWindowFlags_NoMove

        return True

    def hide(self) -> bool:
        if not self._rendering:
            return False

        self._rendering = False
        for handler in self.recon_handlers:
            handler.stop()

        for window in self._windows_list:
            if window['name'] != self._main_window_name:
                window["is_open"].v = False
            window['selected'] = 0

        return True

    def get_templates(self):
        panel_template_copy = deepcopy(self._panel_template)
        self._clean_panel_template_item(panel_template_copy)

        stats_template_copy = deepcopy(self._stats_template)
        self._clean_stats_template(stats_template_copy)
        return {'panel': panel_template_copy, 'stats': stats_template_copy}

    def get_stats_items(self) -> list:
        return self._stats_template['items'].keys()

    def set_panel_font(self, font_id: int):
        self._panel_font = font_id

    def set_stats_font(self, font_id: int):
        self._stats_font = font_id

    def set_stats_data(self, element: str, value: str):
        self._stats_template['items'][element]['value'] = value

    def is_rendering(self) -> bool:
        return self._rendering

    def find_window_by_name(self, name: str):
        return next((x for x in self._windows_list if x['name'] == name), None)

    def render_panel(self):
        for window in self._windows_list:
            if window["is_open"]:
                panel_font = window.get('font', None) or self._panel_font
                if panel_font:
                    ImGui.PushFont(panel_font)
                    self.render_panel_window(window)
                    ImGui.PopFont()
                else:
                    self.render_panel_window(window)

    def render_stats(self):
        window = self._stats_template
        self._set_window_settings(window)
        ImGui.Begin("##RECON_STATS_WINDOW", window["is_open"], self._panel_flags)
        self._save_window_pos_size(window)

        stats_font = window.get('font', None) or self._stats_font
        if stats_font:
            ImGui.PushFont(stats_font)

        ImGui.SetWindowFontScale(self._panel_font_size)
        ImGui.Text(self.player_name)
        ImGui.Text(f'ID: {self.player_id}')
        ImGui.Separator()

        ImGui.BeginTable("##recon_stats_table", 2, ImGuiTableFlags_NoBordersInBody)
        for name, data in window['items'].items():
            ImGui.TableNextRow()
            ImGui.TableNextColumn()
            ImGui.TextColored(ImColor(*data['im_color']), f'{data.get("display", name)}: ')
            ImGui.TableNextColumn()
            ImGui.TextUnformatted(data.get('value', '-'))
        ImGui.EndTable()

        if stats_font:
            ImGui.PopFont()
        ImGui.End()

    def render_panel_window(self, window):
        self._set_window_settings(window)
        ImGui.Begin(window["name"], window["is_open"], self._panel_flags)
        ImGui.SetWindowFontScale(self._panel_font_size)
        selected = window.get('selected', 0)

        self._save_window_pos_size(window)

        for i, (name, value) in enumerate(window.get('items').items()):
            if ImGui.Selectable(name, i == selected):
                window["selected"] = i
                self._select_item(value)

        ImGui.End()

    def _set_window_settings(self, window):
        if window['pos_x'] < 0.0:
            window['pos_x'] = RenderAPI.displayWidth + window['pos_x']
        w_pos = ImVec2(window["pos_x"], window["pos_y"])
        w_size = ImVec2(window["size_x"], window["size_y"])
        ImGui.SetNextWindowPos(w_pos, ImGuiCond_FirstUseEver if self._change_mode else 0)
        ImGui.SetNextWindowSize(w_size, ImGuiCond_FirstUseEver if self._change_mode else 0)

    def _save_window_pos_size(self, window):
        if self._change_mode:
            window_pos = ImGui.GetWindowPos()
            window_size = ImGui.GetWindowSize()
            window['pos_x'] = window_pos.x
            window['pos_y'] = window_pos.y
            window['size_x'] = window_size.x
            window['size_y'] = window_size.y

    def __render(self):
        self.render_panel()
        self.render_stats()

    def _select_item(self, item):
        if isinstance(item, dict):
            self._change_active_window(item['name'])
        elif not self._change_mode:
            if isinstance(item, str):
                callback = self.callbacks.get(item, None)
                if callable(callback):
                    callback(self.player_id)
            elif isinstance(item, int):
                self._textdraw_click(item)
                window = self.find_window_by_name(self._active_window)
                self._change_active_window(window['parent'], True)

    def _textdraw_click(self, textdraw_id: int):
        textdraw_data = BitStream()
        textdraw_data.Write(textdraw_id, TYPE_INT16)
        event_api.send_rpc(RPC_ClickTextDraw, textdraw_data)

    def _scroll(self, down=True):
        current_window = self.find_window_by_name(self._active_window)
        selected = current_window.get('selected', 0)
        if down:
            if len(current_window['items']) != selected + 1:
                current_window['selected'] = selected + 1
            else:
                current_window['selected'] = 0
        else:
            if selected != 0:
                current_window['selected'] = selected - 1
            else:
                current_window['selected'] = len(current_window['items']) - 1

    def _change_active_window(self, new_window_name, close: bool = False):
        new_window = self.find_window_by_name(new_window_name)
        if not new_window:
            return

        if close:
            current_window = self.find_window_by_name(self._active_window)
            current_window['is_open'].v = False
            current_window['selected'] = 0

        new_window['is_open'].v = True
        self._active_window = new_window_name

    def __keycombo_callback(self, keys):
        if keys == 'w' or keys == 'up':
            self._scroll(down=False)
        elif keys == 's' or keys == 'down':
            self._scroll()
        elif keys == 'd' or keys == 'right' or keys == 'lshift' or keys == 'space':
            window = self.find_window_by_name(self._active_window)
            selected = window.get('selected', 0)
            item = list(window['items'].values())[selected]
            self._select_item(item)
        elif keys == 'a' or keys == 'left':
            window = self.find_window_by_name(self._active_window)
            self._change_active_window(window['parent'], True)

    def _spec_hook(self, bsdata, event):
        data = parse_packet(bsdata, event)
        if data['lrAnalog'] in [-128, 128] or data['udAnalog'] in [-128, 128] or data['keys'] == 8:
            return False

    def _init_panel_template(self, item: dict, parent=None):
        parent_for_child = item.get('name', None) or parent
        if 'name' in item:
            item['parent'] = parent
            item['is_open'] = ImBool(parent is None)
            self._windows_list.append(item)

        for name, value in item.items():
            if isinstance(value, list):
                for i in item[name]:
                    if isinstance(i, dict):
                        self._init_panel_template(i, parent_for_child)
            elif isinstance(value, dict):
                self._init_panel_template(item[name], parent_for_child)

    def _clean_panel_template_item(self, item: dict):
        item.pop('is_open', None)
        item.pop('selected', None)
        item.pop('parent', None)

        for name, value in item.items():
            if isinstance(value, list):
                for i in item[name]:
                    if isinstance(i, dict):
                        self._clean_panel_template_item(i)
            elif isinstance(value, dict):
                self._clean_panel_template_item(item[name])

    def _init_stats_template(self):
        self._stats_template['is_open'] = ImBool(True)

        for name, value in self._stats_template['items'].items():
            if 'color' in value:
                value['im_color'] = ColorUtils.hex_to_rgb(value['color'])
            else:
                value['im_color'] = [255, 255, 255, 255]

    @staticmethod
    def _clean_stats_template(template: dict):
        template.pop('is_open', None)
        for name, value in template['items'].items():
            value.pop('im_color', None)
            value.pop('value', None)
