from sobeit.imgui import ImGui

from modsa_lib.core import Chat
from modsa_lib.core.keycombo import KeycomboPressed, KeycomboUpdaterMixin
from modsa_lib.core.command import (
    Command,
    CmdArg,
    CmdArgParser,
    CmdArgInt,
    MinNumberValue,
    MaxNumberValue,
    CmdArgPlayerid,
    CmdArgPlayeridName,
    CommandDocMixin,
)
from modsa_lib.utils.config import ConfigMixin
from modsa_lib.consts.keycombo import VK_OEM_6
from modsa_lib.consts.sobeit import *
from modsa_lib.imgui.fonts import IMGUI_DEFAULT_FONT_19
from modsa_lib.core.sobeit_api import EventAPI, PlayerAPI
from srp_basic.tasks import (
    TaskGarm,
    TaskHHP,
    TaskML,
    TaskSetObj,
    TaskGiveHB,
    TaskOffuninv,
)


class SrpCommands(ConfigMixin, CommandDocMixin, KeycomboUpdaterMixin):
    config_path = 'srp_basic/common.json'

    def __init__(self):
        self.tasks = {}
        self.load_config()

        self.cmd_ags = Command(
            'ags',
            self.callback_ags,
            arg_parser=CmdArgParser({'player': CmdArgPlayeridName()}),
            description='сокращение от /agetstats',
        )
        self.cmd_garm = Command(
            'garm',
            self.callback_garm,
            arg_parser=CmdArgParser(
                {
                    'armour': CmdArgInt(
                        validators=[MinNumberValue(0), MaxNumberValue(255)]
                    )
                }
            ),
            description='массовая выдача брони',
        )
        self.cmd_hhp = Command(
            'hhp',
            self.callback_hhp,
            arg_parser=CmdArgParser(
                {'hp': CmdArgInt(validators=[MinNumberValue(0), MaxNumberValue(255)])}
            ),
            description='массовая выдача хп',
        )
        self.cmd_ml = Command(
            'ml',
            self.callback_ml,
            arg_parser=CmdArgParser(
                {
                    'playerid': CmdArgPlayerid(),
                    'fraction': CmdArgInt(required=False, arg_format_name="ид фракции"),
                }
            ),
            description='назначение игрока на лидерку',
        )
        self.cmd_setobj = Command(
            'setobj',
            self.callback_setobj,
            description='установка объектов из файла на карте',
        )
        self.cmd_givehb = Command(
            'givehb',
            self.callback_givehb,
            arg_parser=CmdArgParser(
                {
                    'playerid': CmdArgPlayerid(required=False),
                    'filename': CmdArg(required=False),
                }
            ),
            description='выдача объектов игроку(ам)',
        )
        self.cmd_offuninv = Command(
            'offuninv',
            self.callback_offuninv,
            description='массовое увольнение игроков из файла',
        )

        self.set_keycombo_handlers(
            {
                "goto_key": KeycomboPressed(
                    self.config.data.get('goto_key'), self.handle_goto_keys
                )
            }
        )

    def init_tasks(self):
        """
        Таски описаны отдельно, так как они должны инициализироваться после EVENT_GAME_INIT
        """
        self.tasks.update(
            {
                'garm': TaskGarm(),
                'hhp': TaskHHP(),
                'ml': TaskML(),
                'setobj': TaskSetObj(),
                'givehb': TaskGiveHB(),
                'offuninv': TaskOffuninv(),
            }
        )

    def get_default_config(self):
        return {'goto_key': VK_OEM_6}

    def callback_ags(self, args, cmd, parser_data):
        Chat.send_text(f'/agetstats {parser_data.get("player")}')

    def callback_garm(self, args, cmd, parser_data):
        task = self.tasks['garm']
        if task.is_running():
            task.stop()
        else:
            task.armour_amount = parser_data['armour']
            task.start()

    def callback_hhp(self, args, cmd, parser_data):
        task = self.tasks['hhp']
        if task.is_running():
            task.stop()
        else:
            task.hp_amount = parser_data['hp']
            task.start()

    def callback_ml(self, args, cmd, parser_data):
        playerid = parser_data.get('playerid')
        fraction = parser_data.get('fraction')
        if fraction:
            return Chat.send_text(f'/makeleader {playerid} {fraction}')

        self.tasks['ml'].show_modal(playerid)

    def callback_setobj(self, args, cmd):
        self.tasks['setobj'].run()

    def callback_givehb(self, args, cmd, parser_data):
        playerid = parser_data.get('playerid')
        filename = parser_data.get('filename')
        self.tasks['givehb'].run(playerid, filename)

    def callback_offuninv(self, args, cmd):
        self.tasks['offuninv'].start()

    def handle_goto_keys(self):
        return Chat.send_text(f"/goto {PlayerAPI.get_id()}")

    def render_settings(self, rendering: bool):
        if rendering:
            ImGui.PushFont(IMGUI_DEFAULT_FONT_19)

            if self.imgui_keycombo("goto_key", "Goto к себе"):
                self.config.data["goto_key"] = self.get_new_keycombo()

            ImGui.PopFont()
        else:
            self.save_config()


srp_commands = SrpCommands()


def on_game_init():
    srp_commands.init_tasks()


EventAPI.register(EVENT_GAME_INIT, on_game_init)
